"""Selection rectangle graphics item."""

from __future__ import annotations

from typing import Optional

from PySide6.QtCore import Qt, QRectF, QPointF
from PySide6.QtGui import QPen, QBrush, QColor
from PySide6.QtWidgets import QGraphicsRectItem


class SelectionRect(QGraphicsRectItem):
    """Measurement rectangle with thin border and resize handles."""

    HANDLE_SIZE = 2
    MIN_SIZE = 2

    def __init__(self, rect: QRectF, page_rect: QRectF, parent=None):
        super().__init__(parent)
        self._page_rect = page_rect
        self._rect = self._clamp(rect.normalized())

        pen = QPen(QColor(91, 141, 246, 220))
        pen.setWidthF(0.5)
        pen.setStyle(Qt.PenStyle.DotLine)
        pen.setCosmetic(True)
        self.setPen(pen)
        self.setBrush(QBrush(QColor(91, 141, 246, 36)))
        self.setZValue(1000)
        self.setAcceptHoverEvents(True)

        self._drag_mode: Optional[str] = None
        self._drag_start: Optional[QPointF] = None
        self._start_rect: Optional[QRectF] = None

        self._sync()

    @property
    def pdf_rect(self) -> QRectF:
        return self._rect

    @pdf_rect.setter
    def pdf_rect(self, r: QRectF):
        self._rect = self._clamp(r.normalized())
        self._sync()

    def _clamp(self, r: QRectF) -> QRectF:
        p = self._page_rect
        x = max(p.left(), min(r.x(), p.right() - self.MIN_SIZE))
        y = max(p.top(), min(r.y(), p.bottom() - self.MIN_SIZE))
        w = max(self.MIN_SIZE, min(r.width(), p.right() - x))
        h = max(self.MIN_SIZE, min(r.height(), p.bottom() - y))
        return QRectF(x, y, w, h)

    def _sync(self):
        self.setRect(self._rect)

    def hit_test(self, pos: QPointF) -> str:
        """Returns 'tl','tr','bl','br','t','b','l','r','move' or ''."""
        r = self._rect
        s = self.HANDLE_SIZE

        def near(a, b):
            return abs(a - b) < s

        on_left = near(pos.x(), r.left())
        on_right = near(pos.x(), r.right())
        on_top = near(pos.y(), r.top())
        on_bottom = near(pos.y(), r.bottom())

        if on_top and on_left:
            return "tl"
        if on_top and on_right:
            return "tr"
        if on_bottom and on_left:
            return "bl"
        if on_bottom and on_right:
            return "br"
        if on_top:
            return "t"
        if on_bottom:
            return "b"
        if on_left:
            return "l"
        if on_right:
            return "r"
        if r.contains(pos):
            return "move"
        return ""

    def start_drag(self, pos: QPointF, mode: str):
        self._drag_mode = mode
        self._drag_start = pos
        self._start_rect = QRectF(self._rect)

    def update_drag(self, pos: QPointF):
        if not self._drag_mode or not self._start_rect:
            return
        d = pos - self._drag_start
        r = QRectF(self._start_rect)
        m = self._drag_mode

        if m == "move":
            r.translate(d)
        else:
            if "l" in m:
                r.setLeft(r.left() + d.x())
            if "r" in m:
                r.setRight(r.right() + d.x())
            if "t" in m:
                r.setTop(r.top() + d.y())
            if "b" in m:
                r.setBottom(r.bottom() + d.y())

        self.pdf_rect = r

    def end_drag(self):
        self._drag_mode = None

    def nudge(self, dx: float, dy: float):
        r = QRectF(self._rect)
        r.translate(dx, dy)
        self.pdf_rect = r

    def resize_by(self, dw: float, dh: float):
        r = QRectF(self._rect)
        r.setWidth(max(self.MIN_SIZE, r.width() + dw))
        r.setHeight(max(self.MIN_SIZE, r.height() + dh))
        self.pdf_rect = r

    def cursor_for_mode(self, mode: str) -> Qt.CursorShape:
        cursors = {
            "tl": Qt.CursorShape.SizeFDiagCursor,
            "br": Qt.CursorShape.SizeFDiagCursor,
            "tr": Qt.CursorShape.SizeBDiagCursor,
            "bl": Qt.CursorShape.SizeBDiagCursor,
            "t": Qt.CursorShape.SizeVerCursor,
            "b": Qt.CursorShape.SizeVerCursor,
            "l": Qt.CursorShape.SizeHorCursor,
            "r": Qt.CursorShape.SizeHorCursor,
            "move": Qt.CursorShape.SizeAllCursor,
        }
        return cursors.get(mode, Qt.CursorShape.ArrowCursor)



from neoview.utils.units import pt_to_mm, pt_to_pica, format_size


def test_pt_to_mm():
    assert abs(pt_to_mm(72.0) - 25.4) < 1e-6


def test_pt_to_pica():
    assert pt_to_pica(12.0) == 1.0


def test_format_size():
    text = format_size(72.0, 36.0)
    assert "W: 72.0pt" in text
    assert "H: 36.0pt" in text



"""Data models for viewer/session state."""

from .view_state import (
    AnnotationRecord,
    BookmarkRecord,
    DocumentSidecarState,
    SearchMatch,
    TabContext,
)

__all__ = [
    "AnnotationRecord",
    "BookmarkRecord",
    "DocumentSidecarState",
    "SearchMatch",
    "TabContext",
]



"""NeoView application theme."""

DARK_STYLE = """
* {
    font-family: "JetBrains Mono", "Fira Code", "Consolas", monospace;
    color: #e8e8ed;
    selection-background-color: #5b8df6;
    selection-color: #ffffff;
}
QMainWindow,
QDialog,
QWidget {
    background: #0d0d0f;
    color: #e8e8ed;
}
QMenuBar {
    background: #111116;
    border-bottom: 1px solid #23242b;
    padding: 4px 8px;
}
QMenuBar::item {
    padding: 5px 10px;
    border-radius: 6px;
}
QMenuBar::item:selected {
    background: #1b2438;
    color: #ffffff;
}
QMenu {
    background: #111116;
    border: 1px solid #242733;
    padding: 6px;
}
QMenu::item {
    padding: 6px 28px 6px 12px;
    border-radius: 6px;
}
QMenu::item:selected {
    background: #20345f;
}
QMenu::separator {
    height: 1px;
    background: #252733;
    margin: 6px 6px;
}
QToolBar {
    background: #101015;
    border: none;
    border-bottom: 1px solid #23242b;
    spacing: 6px;
    padding: 6px 10px;
}
QToolBar::separator {
    width: 1px;
    background: #272a35;
    margin: 3px 10px;
}
QToolButton {
    background: #161822;
    border: 1px solid #272b39;
    border-radius: 7px;
    padding: 6px;
    min-width: 28px;
    min-height: 28px;
}
QToolButton:hover {
    background: #1d2230;
    border-color: #384059;
}
QToolButton:pressed,
QToolButton:checked {
    background: #2a3f71;
    border-color: #5b8df6;
    color: #ffffff;
}
QToolButton::menu-indicator {
    image: none;
    width: 0;
}
QStatusBar {
    background: #0f1014;
    border-top: 1px solid #23242b;
    min-height: 28px;
}
QStatusBar QLabel {
    padding: 0 8px;
    color: #cfd3de;
}
QGraphicsView {
    background: #141417;
    border: none;
}
QScrollBar:vertical {
    background: #0e1015;
    width: 12px;
    margin: 0;
}
QScrollBar::handle:vertical {
    background: #2a2f40;
    border-radius: 6px;
    min-height: 28px;
}
QScrollBar::handle:vertical:hover {
    background: #3b4257;
}
QScrollBar::add-line:vertical,
QScrollBar::sub-line:vertical {
    height: 0;
}
QScrollBar:horizontal {
    background: #0e1015;
    height: 12px;
    margin: 0;
}
QScrollBar::handle:horizontal {
    background: #2a2f40;
    border-radius: 6px;
    min-width: 28px;
}
QScrollBar::handle:horizontal:hover {
    background: #3b4257;
}
QScrollBar::add-line:horizontal,
QScrollBar::sub-line:horizontal {
    width: 0;
}
QLineEdit,
QComboBox,
QPlainTextEdit,
QTextEdit,
QSpinBox {
    background: #151720;
    border: 1px solid #2a2d3a;
    border-radius: 6px;
    padding: 6px 8px;
    color: #e8e8ed;
}
QLineEdit:focus,
QComboBox:focus,
QPlainTextEdit:focus,
QTextEdit:focus {
    border-color: #5b8df6;
}
QComboBox::drop-down {
    border: none;
    width: 18px;
}
QComboBox QAbstractItemView {
    background: #111218;
    border: 1px solid #2b3040;
}
QPushButton {
    background: #1a2235;
    border: 1px solid #2c3e66;
    border-radius: 7px;
    padding: 6px 12px;
    color: #e9eeff;
}
QPushButton:hover {
    background: #223055;
    border-color: #456bb5;
}
QPushButton:pressed {
    background: #182644;
}
QPushButton[secondary="true"] {
    background: #171920;
    border: 1px solid #303342;
    color: #d8dce8;
}
QListWidget,
QTreeWidget {
    background: #111218;
    border: 1px solid #262a36;
    border-radius: 8px;
    padding: 4px;
}
QListWidget::item,
QTreeWidget::item {
    padding: 4px 6px;
    border-radius: 4px;
}
QListWidget::item:selected,
QTreeWidget::item:selected {
    background: #2a3e71;
}
QDockWidget::title {
    background: #101116;
    border: 1px solid #252935;
    border-bottom: none;
    padding: 8px;
    text-align: left;
}
QDockWidget > QWidget {
    border: 1px solid #252935;
    border-top: none;
}
QCheckBox {
    spacing: 7px;
}
QCheckBox::indicator {
    width: 14px;
    height: 14px;
    border: 1px solid #3a4258;
    border-radius: 3px;
    background: #131620;
}
QCheckBox::indicator:checked {
    background: #5b8df6;
    border-color: #5b8df6;
}
QToolTip {
    background: #1a1d27;
    border: 1px solid #3a4668;
    color: #f1f4ff;
    padding: 4px 8px;
}
QFrame[separator="true"] {
    background: #272a35;
    max-height: 1px;
    min-height: 1px;
}
QWidget#InspectorPanel {
    background: #101116;
}
QToolButton#InspectorSectionHeader {
    background: transparent;
    border: none;
    font-weight: 600;
    text-align: left;
    padding: 8px 4px 6px 2px;
}
QToolButton#InspectorSectionHeader:hover {
    color: #ffffff;
}
QWidget#InspectorSectionContent {
    background: transparent;
}
QLabel#InfoLabel {
    color: #b9bfce;
}
QLabel#InfoValue {
    color: #f3f5fb;
    font-weight: 600;
}
QLabel#MonoValue {
    color: #f4f7ff;
    font-size: 14px;
    font-weight: 700;
}
QLabel#StatusDot {
    min-width: 10px;
    max-width: 10px;
    min-height: 10px;
    max-height: 10px;
    border-radius: 5px;
    background: #5b8df6;
}
QLabel#StatusDot[offline="true"] {
    background: #4f5566;
}
QLabel#FloatingMeasureBadge {
    background: rgba(8, 10, 16, 230);
    border: 1px solid rgba(91, 141, 246, 180);
    border-radius: 7px;
    padding: 4px 7px;
    color: #e9f0ff;
    font-size: 12px;
    font-weight: 700;
}
QLabel#LinkBadge {
    background: rgba(8, 10, 16, 230);
    border: 1px solid rgba(91, 141, 246, 180);
    border-radius: 9px;
}
"""

LIGHT_STYLE = """
* {
    font-family: "Segoe UI", "Noto Sans", "Arial", sans-serif;
    color: #1f2430;
    selection-background-color: #2f6feb;
    selection-color: #ffffff;
}
QMainWindow,
QDialog,
QWidget {
    background: #f6f8fb;
    color: #1f2430;
}
QMenuBar {
    background: #ffffff;
    border-bottom: 1px solid #dce4ef;
    padding: 4px 8px;
}
QMenuBar::item {
    padding: 5px 10px;
    border-radius: 6px;
}
QMenuBar::item:selected {
    background: #eaf1ff;
    color: #12356f;
}
QMenu {
    background: #ffffff;
    border: 1px solid #d6deeb;
    padding: 6px;
}
QMenu::item {
    padding: 6px 28px 6px 12px;
    border-radius: 6px;
}
QMenu::item:selected {
    background: #e9f0ff;
}
QMenu::separator {
    height: 1px;
    background: #dde4ef;
    margin: 6px 6px;
}
QToolBar {
    background: #ffffff;
    border: none;
    border-bottom: 1px solid #dce4ef;
    spacing: 6px;
    padding: 6px 10px;
}
QToolBar::separator {
    width: 1px;
    background: #d8e0ec;
    margin: 3px 10px;
}
QToolButton {
    background: #f8faff;
    border: 1px solid #d7e0ee;
    border-radius: 7px;
    padding: 6px;
    min-width: 28px;
    min-height: 28px;
}
QToolButton:hover {
    background: #eef4ff;
    border-color: #bfceed;
}
QToolButton:pressed,
QToolButton:checked {
    background: #dbe8ff;
    border-color: #2f6feb;
    color: #12356f;
}
QToolButton::menu-indicator {
    image: none;
    width: 0;
}
QStatusBar {
    background: #ffffff;
    border-top: 1px solid #dce4ef;
    min-height: 28px;
}
QStatusBar QLabel {
    padding: 0 8px;
    color: #4a5a71;
}
QGraphicsView {
    background: #eef2f7;
    border: none;
}
QScrollBar:vertical {
    background: #eff3f9;
    width: 12px;
    margin: 0;
}
QScrollBar::handle:vertical {
    background: #c8d4e6;
    border-radius: 6px;
    min-height: 28px;
}
QScrollBar::handle:vertical:hover {
    background: #b5c5dc;
}
QScrollBar::add-line:vertical,
QScrollBar::sub-line:vertical {
    height: 0;
}
QScrollBar:horizontal {
    background: #eff3f9;
    height: 12px;
    margin: 0;
}
QScrollBar::handle:horizontal {
    background: #c8d4e6;
    border-radius: 6px;
    min-width: 28px;
}
QScrollBar::handle:horizontal:hover {
    background: #b5c5dc;
}
QScrollBar::add-line:horizontal,
QScrollBar::sub-line:horizontal {
    width: 0;
}
QLineEdit,
QComboBox,
QPlainTextEdit,
QTextEdit,
QSpinBox {
    background: #ffffff;
    border: 1px solid #cfd9e8;
    border-radius: 6px;
    padding: 6px 8px;
    color: #1f2430;
}
QLineEdit:focus,
QComboBox:focus,
QPlainTextEdit:focus,
QTextEdit:focus {
    border-color: #2f6feb;
}
QComboBox::drop-down {
    border: none;
    width: 18px;
}
QComboBox QAbstractItemView {
    background: #ffffff;
    border: 1px solid #d2dced;
}
QPushButton {
    background: #2f6feb;
    border: 1px solid #285fcc;
    border-radius: 7px;
    padding: 6px 12px;
    color: #ffffff;
}
QPushButton:hover {
    background: #255fd0;
    border-color: #1e53ba;
}
QPushButton:pressed {
    background: #1f52b7;
}
QPushButton[secondary="true"] {
    background: #f1f5fc;
    border: 1px solid #ccd7e9;
    color: #2c3a50;
}
QListWidget,
QTreeWidget {
    background: #ffffff;
    border: 1px solid #d2dced;
    border-radius: 8px;
    padding: 4px;
}
QListWidget::item,
QTreeWidget::item {
    padding: 4px 6px;
    border-radius: 4px;
}
QListWidget::item:selected,
QTreeWidget::item:selected {
    background: #dce8ff;
}
QDockWidget::title {
    background: #f6f9fe;
    border: 1px solid #d7e0ed;
    border-bottom: none;
    padding: 8px;
    text-align: left;
}
QDockWidget > QWidget {
    border: 1px solid #d7e0ed;
    border-top: none;
}
QCheckBox {
    spacing: 7px;
}
QCheckBox::indicator {
    width: 14px;
    height: 14px;
    border: 1px solid #b7c6dd;
    border-radius: 3px;
    background: #ffffff;
}
QCheckBox::indicator:checked {
    background: #2f6feb;
    border-color: #2f6feb;
}
QToolTip {
    background: #ffffff;
    border: 1px solid #bfcde4;
    color: #1f2430;
    padding: 4px 8px;
}
QFrame[separator="true"] {
    background: #dbe3ef;
    max-height: 1px;
    min-height: 1px;
}
QWidget#InspectorPanel {
    background: #f9fbff;
}
QToolButton#InspectorSectionHeader {
    background: transparent;
    border: none;
    font-weight: 600;
    text-align: left;
    padding: 8px 4px 6px 2px;
}
QToolButton#InspectorSectionHeader:hover {
    color: #12356f;
}
QWidget#InspectorSectionContent {
    background: transparent;
}
QLabel#InfoLabel {
    color: #5c6a82;
}
QLabel#InfoValue {
    color: #1f2430;
    font-weight: 600;
}
QLabel#MonoValue {
    color: #1e2e46;
    font-size: 14px;
    font-weight: 700;
}
QLabel#StatusDot {
    min-width: 10px;
    max-width: 10px;
    min-height: 10px;
    max-height: 10px;
    border-radius: 5px;
    background: #2f6feb;
}
QLabel#StatusDot[offline="true"] {
    background: #8391a8;
}
QLabel#FloatingMeasureBadge {
    background: rgba(255, 255, 255, 232);
    border: 1px solid rgba(47, 111, 235, 170);
    border-radius: 7px;
    padding: 4px 7px;
    color: #1f2a3d;
    font-size: 12px;
    font-weight: 700;
}
QLabel#LinkBadge {
    background: rgba(255, 255, 255, 232);
    border: 1px solid rgba(47, 111, 235, 170);
    border-radius: 9px;
}
"""
